'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * Copyright (c) 2015-present, Zippy Technologies
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/**
  getPositions returns positions considering only these two systems:

   +------------->   +------------>
   |                 |      d
   |                 |
   | d               |
   |                 |
   |                 |
   v                 v

  bottom - right and right - bottom


  From this sistem we can calculate the next ones.


  +-------------------------+
  | Stacking:               |   ^                  ^
  | top-right and right-top |   |d                 |
  |                         |   |                  |
  | left` = left            |   |                  |
  | bottom` = top           |   |                  |
  +-------------------------+   +-------------->   +--------------->
                                                        d
  +-------------------------+    <-----------+       <-----------+
  | Stacking:               |          d     |                   |
  | left-bottom and bottom-left |            |                   |
  |                         |                |                 d |
  | top` = top              |                |                   |
  | right` = left           |                |                   v
  +---------------+---------+                v

  +-------------------------+
  | Stacking:               |              ^                    ^
  | left-top and top-left   |              |                    |
  |                         |              |                 d  |
  | bottom` = top           |              |                    |
  | right` = left           |        d     |                    |
  +----------------+--------+   <----------+        <-----------+

  d = dominant
  first direction of stacking is dominant. First it stacks
  in that direction until it fits and it stacks in the next direction.
 */

function handleTopRight(_ref) {
  var top = _ref.top,
      left = _ref.left,
      offset = _ref.offset;

  return {
    offset: offset,
    left: left,
    bottom: top
  };
}

function handleDownLeft(_ref2) {
  var top = _ref2.top,
      left = _ref2.left,
      offset = _ref2.offset;

  return {
    offset: offset,
    top: top,
    right: left
  };
}

function handleTopLeft(_ref3) {
  var left = _ref3.left,
      top = _ref3.top,
      offset = _ref3.offset;

  return {
    offset: offset,
    bottom: top,
    right: left
  };
}

var topRightPositions = {
  'top-right': true,
  'right-top': true,
  top: true // has same formula
};

var downLeftPositions = {
  'left-bottom': true,
  'bottom-left': true,
  left: true // has same formula
};

var topLeftPositions = {
  'top-left': true,
  'left-top': true
};

function translatePositions(_ref4) {
  var positions = _ref4.positions,
      stacking = _ref4.stacking;

  var newPositions = positions;
  if (topRightPositions[stacking]) {
    newPositions = positions.map(handleTopRight);
  }

  if (downLeftPositions[stacking]) {
    newPositions = positions.map(handleDownLeft);
  }

  if (topLeftPositions[stacking]) {
    newPositions = positions.map(handleTopLeft);
  }

  return newPositions;
}

exports.default = translatePositions;